      program scf2forces
!
!     Evgeniya Kabliman   09 Dec.2010
!     Evgeniya Kabliman   10 May 2011
!     Institute of Materials Chemistry
!     Vienna University of Technology
!     evgeniya@theochem.tuwien.ac.at
!
! 1. 'scf2forces' program applies the symmetry operations to forces from case.scf 
!     and creates 'FORCES_SETS' file for 'phonopy' program (phonons calc.).
! 2.  'phonopy' limit = works only with P1 symmetry
! 3.  'wien2k' efficiency = converts supercell structures created by 'phonopy' 
!     to structures with higher symmetry
!
      use struct
      use symetr
      use defs
!
      implicit none
      integer :: i, j, k, l, m, q, ndisp, npos, nd, nx, ny, nz, natoms 
      integer :: where1, where2, nindex, lnam, lnam2, length 
      real*16, allocatable :: forces(:,:),forces_new(:,:),pos1(:,:),ampl(:) 
      character*100 :: line, line1, fname, fname1, fname2 
      character*80 :: opennumber, opennumber1, opennumber2, text
!  
      open(1,file='disp.yaml',status='old',err=1001) 
      open(2,file='FORCE_SETS',err=1011) 
!
      write(*,'(a)')'Please, type name of your &  
      nameXXX.struct(scf) files:'
      write(*,'(a)')'(XXX=001,002,...,099,...,999 &
      supercell calculations)' 
      read(*,'(a)')fname
      lnam = len(trim(fname))
      write(*,'(a)')'Please, type name of name.structS-XXX & 
      created by PHONOPY:'
      read(*,'(a)')fname2
      lnam2 = len(trim(fname2))
!
! reading disp.yaml file
      read(1,'(6x,i5)')natoms  
      read(1,*)  
      ndisp=0
      do
        read(1,'(a)',end=101,err=1001)line          
        length=len(trim(line))
        line1=adjustl(line)
        where1=index(line1,'lattice:')
!        if(where1.ne.0) goto 200
        if(where1.ne.0) exit
        ndisp = ndisp + 1
      enddo
101   continue
200   ndisp = ndisp/5 ! 5 lines for each displacement
      rewind(1)
      read(1,*)
      read(1,*)
!
      allocate(ampl(3))
      write(*,*) 
      write(*,'(a25,i3)')'number of displacements =',ndisp
      write(text,*)natoms
      write(2,'(a)')trim(adjustl(text))
      write(text,*)ndisp
      write(2,'(a)')trim(adjustl(text))
      do i = 1,ndisp 
        if(i.lt.10) then
          write(opennumber,'("00",i1,".struct")') i
          write(opennumber1,'("00",i1,".scf")') i
          write(opennumber2,'(".structS-00",i1)') i
        elseif(i.lt.100) then
          write(opennumber,'("0",i2,".struct")') i
          write(opennumber1,'("0",i2,".scf")') i
          write(opennumber2,'(".structS-0",i2)') i
        else
          write(opennumber,'(i3,".struct")') i
          write(opennumber1,'(i3,".scf")') i
          write(opennumber2,'(".structS-",i3)') i
        endif
        fname = fname(1:lnam)//opennumber
        fname1 = fname(1:lnam)//opennumber1
        fname2 = fname2(1:lnam2)//opennumber2
        write(*,'(2a20)')fname,fname1
        open(20,file=fname,status='old',form='formatted',err=1002)
        open(10,file=fname1,status='old',form='formatted',err=1003)
        open(3,file='temp-forces.out')
!
! reading structdata from *.struct
        call readstruct 
        allocate(forces(3,nat))
        allocate(forces_new(3,ndif))
        allocate(pos1(3,ndif))
        forces = 0.0d0
        forces_new = 0.0d0
        pos1 = pos       
!
! creating a displacements
        read(1,'(9x,i3)')nd 
        read(1,*)           
        read(1,*)           
        read(1,*)           
        read(1,'(6x,f20.16,x,f20.16,x,f20.16)')ampl(1),ampl(2),ampl(3)
        write(text,*)nd
        write(2,*)      
        write(2,'(a,/,3f20.16)')trim(adjustl(text)),ampl(1),ampl(2),& 
          ampl(3) 
        write(*,'(i3,3f20.16)')nd,ampl(1),ampl(2),ampl(3)  
!
! reading forces from *.scf and write them to scf-forces.out
        do
          read(10,'(A)',end=102,err=1003)line
          length=len(trim(line))
          line1=adjustl(line)
          where1=index(line1,'FGL')
          where2=index(line1,'total forces')
          if (where1.ne.0.and.where2.ne.0) then
            write(3,'(A)')line(1:length)
          endif
        enddo
102     continue
        rewind(3)
!
        call rotdef(iz,tau,iord,nat,pos,ndif,rotij,tauij,mult,lattic) 
        nindex = 0
        do j = 1,nat
          read(3,'(29X,3F16.9)')forces(1:3,j)
          do k = 1,mult(j)
            nindex=nindex+1
            forces_new(1:3,nindex)=MATMUL(forces(1:3,j), &
                               rotij(1:3,1:3,nindex))
!            write(2,'(3f16.10)')(forces_new(m,nindex),m=1,3)
          enddo 
        enddo 
        call deallocatestruct
        close(3)
        close(10)
        close(20)
!     
        open(20,file=fname2,status='old',form='formatted',err=1002)
        call readstruct
        do k = 1,ndif
          do l = 1,ndif
            npos = 0
            do q = 1,3     
            if(pos(q,k).eq.pos1(q,l))then
              npos = npos + 1
            endif
            if(npos.eq.3) then 
              write(2,'(3f15.10)')(forces_new(m,l),m=1,3)
            endif 
            enddo
          enddo
        enddo
        deallocate(forces,forces_new,pos1)
        call deallocatestruct 
        close(20)          
      enddo
      stop 
!
 1001 write(*,*)'error in opening DISP'
 1011 write(*,*)'error in opening FORCES'
 1002 write(*,*)'error in opening *.struct'
 1003 write(*,*)'error in opening *.scf'
! 
      close(1)
      close(2)
      end
!
!
!    iz(3,3,iord) - symmetry operations (matrixes in the end of "*.struct")
!    tau(3,iord) - translational vectors (vectors in the end of "*.struct")
!    iord - number of symmetry operations (number in the end of "*.struct")
!    nat - number of inequivalent atoms
!    pos1(3,ndif) - coordinates (x,y,z) of all atoms
!    ndif - total number of atoms
!    rotij(3,3,ndif) - rotational matrix
!    tauij(3,ndif) - translational matrix
!    mult(nat) - multiplicity of inequivalent atoms
!    lattic - type of lattice
!
!
subroutine deallocatestruct
    use struct
    use symetr
    deallocate (rmt,iatnr,mult,isplit)
    deallocate (rotloc,r0,dx,jri)
    deallocate (aname,zz,pos)
    if(iord.gt.0) then 
      deallocate (iz,tau,inum)
      deallocate (rotij,tauij)
    endif
end subroutine deallocatestruct 
